      SUBROUTINE ice_bio_sms(nlay_i,kideb,kiut)
!
!-----------------------------------------------------------------------------!
!                           *** ice_bio_sms ***
!     Biological sources minus sinks
!     (c) Martin Vancoppenolle, LOCEAN, October 2012
!     
!     version : source_3.03
!     status  : in permanent revision :-)
!
!=============================================================================!
!
      INCLUDE 'type.com'
      INCLUDE 'para.com'
      INCLUDE 'const.com'
      INCLUDE 'ice.com'
      INCLUDE 'thermo.com'
      INCLUDE 'bio.com'

      INTEGER, DIMENSION(3) :: 
     &   zindex

      REAL, DIMENSION(nlay_bio) ::
     &   zek,
     &   zlim_nut

      LOGICAL ::
     &   ln_write_bio

      ln_write_bio = .TRUE.
      zeps = 1.0e-14
      ji = 1

!
!=============================================================================!
!
!-----------------------------------------------------------------------------!
! 1) Control prints
!-----------------------------------------------------------------------------!
!
      IF ( ln_write_bio ) THEN

         WRITE(numout,*) 
         WRITE(numout,*) ' ** ice_bio_sms : '
         WRITE(numout,*) ' ~~~~~~~~~~~~~~~~ '
         WRITE(numout,*)

         CALL ice_brine

         WRITE(numout,*) ' Initial tracer values   '
         WRITE(numout,*) ' ~~~~~~~~~~~~~~~~~~~~~~  '

         DO jn = 1, ntra_bio
            IF ( flag_active(jn) ) THEN
               WRITE(numout,*) ' Tracer :   ', biotr_i_nam(jn)
               WRITE(numout,*) ' cbu_i_bio: ', ( cbu_i_bio(jn,layer), 
     &                                         layer = 1, nlay_bio )
            ENDIF ! flag_active
         END DO ! jn

         WRITE(numout,*)
         WRITE(numout,*) ' Parameters '
         WRITE(numout,*) ' ~~~~~~~~~~~~~~~~~~~~~~  '
         WRITE(numout,*) ' si_c      : ', si_c
         WRITE(numout,*) ' no3_c     : ', no3_c
         WRITE(numout,*) ' po4_c     : ', po4_c
         WRITE(numout,*) ' mumax_bio : ', mumax_bio   
         WRITE(numout,*) ' klys_bio  : ', klys_bio
         WRITE(numout,*) ' khs_si_bio: ', khs_si_bio
         WRITE(numout,*) ' ek_bio    : ', ek_bio
         WRITE(numout,*) ' astar_alg : ', astar_alg
         WRITE(numout,*) ' rg_bio    : ', rg_bio
         WRITE(numout,*) ' nn_lim_sal: ', nn_lim_sal 
         WRITE(numout,*) ' lim_sal_wid :', lim_sal_wid
         WRITE(numout,*) ' lim_sal_smax:', lim_sal_smax
         WRITE(numout,*) 

      ENDIF ! ln_write_bio
!
!-----------------------------------------------------------------------------!
! 2) Source Minus Sink terms
!-----------------------------------------------------------------------------!
!
      consN = 0

      DO layer = layer_00, nlay_bio 
         
         trucN = cbu_i_bio(jn_din,layer) + cbu_i_bio(jn_eon,layer)
     &              + cbu_i_bio(jn_aon,layer) + consN
         consN    = trucN        

      END DO

      WRITE(numout,*) ' Conservation BIO N :', consN


      ! Reference of the upper layer
      IF ( ( c_grid .EQ. 'SL' ) .OR. ( c_grid .EQ. 'BA' ) ) THEN

         lim_dsi(:) = 0.; lim_no3(:) = 0.; lim_po4(:) = 0.
         lim_lig(:) = 0.; lim_tem(:) = 0.; lim_sal(:) = 0.
         lys_bio(:) = 0.; rem_bio(:) = 0.; syn_bio(:) = 0.
         rsp_bio(:) = 0.; exu_bio(:) = 0.

         lim_dsi(nlay_bio) = 1.; lim_no3(nlay_bio) = 1.
         lim_po4(nlay_bio) = 1.
         lim_lig(nlay_bio) = 1.; lim_tem(nlay_bio) = 1.
         lim_sal(nlay_bio) = 1.

      ELSE

         lim_dsi(:) = 1.; lim_no3(:) = 1.; lim_po4(:) = 1.
         lim_lig(:) = 1.; lim_tem(:) = 1.; lim_sal(:) = 1.
         lys_bio(:) = 0.; rem_bio(:) = 0.; syn_bio(:) = 0.
         rsp_bio(:) = 0.; exu_bio(:) = 0.

      ENDIF

      !------------------------------------------------------------------------
      ! 2.1 Limitation factors
      !------------------------------------------------------------------------

      ! 2.1.1 Nutrients & temperature
      !--------------------------------
      DO layer = layer_00, nlay_bio

         IF ( ln_lim_dsi ) ! DSi limitation
     &      lim_dsi(layer) = c_i_bio(1,layer) 
     &                     / ( khs_si_bio + c_i_bio(1,layer) )

         IF ( ln_lim_no3 ) ! NO3 limitation
     &      lim_no3(layer) = c_i_bio(2,layer) 
     &                     / ( khs_n_bio + c_i_bio(2,layer) )

         IF ( ln_lim_po4 ) ! PO4 limitation
     &      lim_po4(layer) = c_i_bio(3,layer) 
     &                     / ( khs_p_bio + c_i_bio(3,layer) )

         IF ( ln_lim_dsi .OR. ln_lim_no3 .OR. ln_lim_po4 ) ! Nut limitation
!    &      zlim_nut(layer) = lim_dsi(layer) * lim_no3(layer)
!    &                      * lim_po4(layer)
     &      zlim_nut(layer) = MIN ( lim_dsi(layer) , lim_no3(layer)
     &                      , lim_po4(layer) )

         IF ( ln_lim_tem ) ! Temperature factor
     &      lim_tem(layer) = EXP( rg_bio * tc_bio(layer) )

      END DO ! layer

      ! 2.1.2 Brine salinity
      !----------------------
      IF ( ln_lim_sal ) THEN

         SELECT CASE ( nn_lim_sal )

            CASE (1) ! Arrigo and Sullivan (1992)

            DO layer = layer_00, nlay_bio
               zarg = 2.16 - 8.30e-5 * sbr_bio(layer)**2.11
     &              - 0.55 * LOG( sbr_bio(layer) )
               lim_sal(layer) = EXP ( - zarg * zarg )
            END DO

            CASE (2) ! Simple Gaussian ( Martouf and Grozny, 2010 )
            
            DO layer = layer_00, nlay_bio
                zarg = ( sbr_bio(layer) - lim_sal_smax ) /  lim_sal_wid
                lim_sal(layer) = EXP( - zarg * zarg )
            END DO

         END SELECT

      ENDIF ! ln_lim_sal

      ! 2.1.3 Light
      !-------------
      IF ( ln_lim_lig ) THEN

         SELECT CASE ( nn_phs )

            CASE (1) ! prescribed Ek

               zek(:) = ek_bio
               chlC_bio(:) = chla_c

            CASE (2) ! Ek depends on chla/C

               zek(:) = mumax_bio / ( alpha_bio * chla_c )
               chlC_bio(:) = chla_c

            CASE (3) ! full account for limitation factors

               zek(:) = mumax_bio * lim_tem(:) * lim_sal(:) 
     &                * zlim_nut(:) / ( alpha_bio * chla_c ) 
               chlC_bio(:) = chla_c

            CASE (4) ! varying chl-a carbon ratio

               zthmin = 0.01  ! min
               zthmax = 0.05  ! max 

               DO layer = layer_00, nlay_bio
                  chlC_bio (layer) = ( zthmax - ( zthmax - zthmin ) * 
     &            MIN( par_bio(layer) / Estar, 1.0 ) ) * zlim_nut(layer)
               END DO

               zek(:) = mumax_bio * lim_tem(:) * lim_sal(:)
     &                * zlim_nut(:) / ( alpha_bio * chlC_bio(:) )

         END SELECT
         
         DO layer = layer_00, nlay_bio ! Jassby & Platt 76 formulation
            lim_lig(layer) = TANH ( par_bio(layer) / zek(layer) )
         END DO

         WRITE(numout,*)
         WRITE(numout,*) ' lim_lig : ', ( lim_lig(layer), 
     &                   layer = layer_00, nlay_bio )
         WRITE(numout,*) ' zlim_nut: ', ( zlim_nut(layer), 
     &                   layer = layer_00, nlay_bio )
         WRITE(numout,*) ' par_bio : ', ( par_bio(layer), 
     &                   layer = layer_00, nlay_bio )
         WRITE(numout,*) ' zek     : ', ( zek(layer)    , 
     &                   layer = layer_00, nlay_bio )
         WRITE(numout,*) ' chlC_bio: ', ( chlC_bio(layer), 
     &                   layer = layer_00, nlay_bio )
         WRITE(numout,*)

      ENDIF ! ln_lim_lig

      IF ( .NOT. ln_lim_lig ) chlC_bio(:) = chla_c

      !------------------------------------------------------------------------
      ! 2.2 Loss & remineralization 
      !------------------------------------------------------------------------

      ! Exudation is zero
      exu_bio(:) = 0.0

      DO layer = layer_00, nlay_bio

         ! Respiration (NP, NPDr)
         ! IF ( ln_rsp ) ...
         rsp_bio(layer) = krsp_bio * cbu_i_bio(jn_aoc,layer) * 
     &                    lim_tem(layer)

         ! Lysis (NP, NPDr) 
         IF ( ln_lys ) THEN

            IF ( nn_lys .GE. 1 )   ! no T-dependence
     &         lys_bio(layer) = klys_bio * cbu_i_bio(jn_aoc,layer)

            IF ( nn_lys .EQ. 2 )   ! T-dependence
     &         lys_bio(layer) = lys_bio(layer) * lim_tem(layer)

         ENDIF

      END DO

      DO layer = layer_00, nlay_bio

         IF ( ln_rem ) THEN

            SELECT CASE ( nn_bio_opt ) 

               CASE(0) ! NP

                  rem_bio(layer) = frem_bio * 
     &                             lys_bio(layer)

               CASE(1) ! NPD
 
                  IF ( nn_rem .EQ. 1 )   ! no T-dependence
     &               rem_bio(layer) = krem_bio * 
     &                                cbu_i_bio(jn_eoc,layer)

                  IF ( nn_rem .EQ. 2 )   ! T-dependence
     &               rem_bio(layer) = krem_bio * cbu_i_bio(jn_eoc,layer)
     &                                * EXP( rg_bac * tc_bio(layer) )
     
            END SELECT

         ENDIF

      END DO

      !------------------------------------------------------------------------
      ! 2.3 Diatom Synthesis
      !------------------------------------------------------------------------
      zeps = 0.000000001

      DO layer = layer_00, nlay_bio

         IF ( ln_syn ) THEN
            ! raw synthesis (in s-1)
            z_syn = mumax_bio
     &            * lim_tem(layer) * lim_sal(layer) * lim_lig(layer)
     &            * zlim_nut(layer)

            ! preclude excessive growth if insufficient nutrient stock
            zsyn1 = z_syn * cbu_i_bio(jn_aoc,layer) ! normal carbon production rate (mmolC/m3/s)
            zsyn2 = zsyn1;    zsyn3 = zsyn1;    zsyn4 = zsyn1

   
            IF ( ln_lim_dsi )                  ! prevent exhaustion of Si
     &         zsyn2 = cbu_i_bio(jn_dsi,layer) / ddtb / si_c - zeps 
  
            IF (ln_decoupNC) THEN ! C aussi limité en stock par DIN
               zC  = MAX (cbu_i_bio(jn_aoc,layer) , zeps )
               zN  = MAX (cbu_i_bio(jn_aon,layer) , zeps )
               N_C_alg(layer)  = zN / zC
!              N_C_alg(layer) = 0.14 
               zsyn3 = cbu_i_bio(jn_din,layer) / (ddtb*N_C_alg(layer))
     &                 - zeps
            ELSE 
               zsyn3 = cbu_i_bio(jn_din,layer) / (ddtb * no3_c) - zeps
            ENDIF


            IF (ln_decoupPC) THEN ! C aussi limité en stock par DIP
               zC  = MAX (cbu_i_bio(jn_aoc,layer) , zeps )
               zP  = MAX (cbu_i_bio(jn_aop,layer) , zeps )
               P_C_alg(layer)  = zP / zC
!              P_C_alg(layer) = 0.0094 
               zsyn4 = cbu_i_bio(jn_dip,layer)/(ddtb * P_C_alg(layer))
     &                 - zeps
            ELSE
               zsyn4 = cbu_i_bio(jn_dip,layer) / ddtb / po4_c - zeps
            ENDIF


             ! nutrient stock-limited PP rate
            syn_bio(layer) = MIN( zsyn1, zsyn2, zsyn3, zsyn4 ) 

         ELSE  ! ln_syn is .FALSE. 
               ! PP is prescribed to zero everywhere 
               ! except in the bottom layer
               ! where it equals pp_presc
               ! if sun is shining

            IF ( ( layer .EQ. nlay_bio ) .AND. ( fsolgb(ji) .GT. 5.0 ) )
     &         syn_bio(layer) = pp_presc 

         ENDIF ! ln_syn
   

         ! -------------------------
         ! -- Limitations en stocks
         ! ------------------------

         IF (ln_decoupNC) THEN
             zzdin = syn_bio(layer) / (((1/N_C_alg(layer)) *
     &               (cbu_i_bio(jn_din,layer)/ddtb)) - zeps)

         ELSE 
             zzdin = syn_bio(layer) / (((1/no3_c) *
     &               (cbu_i_bio(jn_din,layer)/ddtb)) - zeps)
         ENDIF


         IF (ln_decoupPC) THEN
             zzpo4 = syn_bio(layer) / (((1/P_C_alg(layer)) *
     &               (cbu_i_bio(jn_dip,layer)/ddtb)) - zeps)

         ELSE
             zzpo4 = syn_bio(layer) / (((1/po4_c) *
     &               (cbu_i_bio(jn_dip,layer)/ddtb)) - zeps)
         ENDIF



         zzsi  = syn_bio(layer) / (((1/si_c) *
     &           (cbu_i_bio(jn_dsi,layer)/ddtb)) - zeps)


         IF ( zzdin .EQ. 1 ) THEN
            lim_din_stock(layer) = 1   !DIN est limitant en stocks
         ELSE
            lim_din_stock(layer) = 0   !DIN PAS limitant en stocks
         ENDIF

         IF ( zzpo4 .EQ. 1 ) THEN
            lim_dip_stock(layer) = 1
         ELSE
            lim_dip_stock(layer) = 0
         ENDIF

         IF ( zzsi .EQ. 1) THEN
            lim_dsi_stock(layer) = 1
         ELSE
            lim_dsi_stock(layer) = 0
         ENDIF


      !------------------------------------------------------------------------
      ! N cycle 
      !------------------------------------------------------------------------


         IF ( ln_decoupNC ) THEN

            !--- Nitrogen carbon algal ratio
            zC    = MAX( cbu_i_bio(jn_aoc,layer) , zeps )
            zN    = MAX( cbu_i_bio(jn_aon,layer) , zeps )
            N_C_alg(layer)  = zN / zC
!           zN_C  = MIN( cbu_i_bio(jn_aon,layer) / zC , N_C_max )
!           zN_C  = MAX( N_C_min , zN_C )
!           N_C_alg(layer) = zN_C  ! => marche pas
!            N_C_alg(layer) = 0.14

            !--- Nitrogen carbon detritic ratio
            zC    = MAX( cbu_i_bio(jn_eoc,layer), zeps )
            zNd   = MAX( cbu_i_bio(jn_eon,layer), zeps ) 
            N_C_det(layer) = zNd / zC
!            N_C_det(layer) = 0.14

            !--- Process rates
            syn_N(layer) = N_C_alg(layer) * ksyn_N * syn_bio(layer)
            rsp_N(layer) = N_C_alg(layer) * krsp_N * rsp_bio(layer)
            lys_N(layer) = N_C_alg(layer) * klys_N * lys_bio(layer)
            rem_N(layer) = N_C_det(layer) * krem_N * rem_bio(layer)


         ENDIF ! ln_decoupNC

      !------------------------------------------------------------------------
      ! P cycle 
      !------------------------------------------------------------------------


         IF ( ln_decoupPC ) THEN

            !--- P carbon algal ratio
            zC    = MAX( cbu_i_bio(jn_aoc,layer) , zeps )
            zP    = MAX( cbu_i_bio(jn_aop,layer) , zeps )
            P_C_alg(layer)  = zP / zC

            !--- P carbon detritic ratio
            zC    = MAX( cbu_i_bio(jn_eoc,layer), zeps )
            zPd   = MAX( cbu_i_bio(jn_eop,layer), zeps )
            P_C_det(layer) = zPd / zC

            !--- Process rates
            syn_P(layer) = P_C_alg(layer) * ksyn_P * syn_bio(layer)
            rsp_P(layer) = P_C_alg(layer) * krsp_P * rsp_bio(layer)
            lys_P(layer) = P_C_alg(layer) * klys_P * lys_bio(layer)
            rem_P(layer) = P_C_det(layer) * krem_P * rem_bio(layer)

         ENDIF ! ln_decoupPC


         zPa   = MAX( cbu_i_bio(jn_aop,layer) , zeps )
         zNt   = MAX( cbu_i_bio(jn_aon,layer) , zeps )
         N_P_alg(layer)  = zNt / zPa
         zPe   = MAX( cbu_i_bio(jn_eop,layer) , zeps )
         zNtd  = MAX( cbu_i_bio(jn_eon,layer) , zeps )
         N_P_det(layer)  = zNtd / zPe



      !------------------------------------------------------------------------
      ! 2.4 Update reservoirs (including N cycle reservoirs)
      !------------------------------------------------------------------------

        ! -------------------------------------------------------
        ! -- 1) Increments
        ! ------------------------------------------------------


         ! DSi uptake
         zd_dsi1 = si_c * ( rem_bio(layer) + 
     &             rsp_bio(layer) - syn_bio(layer) ) * ddtb
         zd_dsi2 = - cbu_i_bio(jn_dsi,layer)        ! Maximum DSi uptake
         zd_dsi  = MAX ( zd_dsi1, zd_dsi2 )

         ! update algae
         zd_daf1 = ( syn_bio(layer) - lys_bio(layer) -
     &               rsp_bio(layer) - exu_bio(layer) ) * ddtb
         zd_daf2 = - cbu_i_bio(jn_aoc,layer)        ! Max algal carbon loss
         zd_daf = MAX ( zd_daf1, zd_daf2 )

         ! update detritus
         IF ( nn_bio_opt .GE. 1 ) THEN
            zd_eoc1 = ( lys_bio(layer) + exu_bio(layer) - 
     &                  rem_bio(layer) ) * ddtb
            zd_eoc2 = - cbu_i_bio(jn_eoc,layer)     ! Max detrital carbon loss
            zd_eoc  = MAX( zd_eoc1, zd_eoc2 )
         ENDIF ! nn_bio_opt


         IF (ln_decoupNC) THEN

            !--- Increments
            zd_din1 = ( rsp_N(layer) - syn_N(layer) +
     &                  rem_N(layer) ) * ddtb
            zd_din2 = - cbu_i_bio(jn_din,layer) !PEU PAS EPUISER PLUS
                                                !QUE CE QUIL Y A !!!!
            zd_din  = MAX ( zd_din1, zd_din2 )  ! interessant dans le cas
                                                ! ou zd_din1 est négatif
                                                ! zd_din1 peut pas être
                                                ! plus petit que zd_din2 

            zd_aon1 = ( syn_N(layer) - rsp_N(layer) - lys_N(layer) ) *
     &                ddtb
            zd_aon2 = - cbu_i_bio(jn_aon,layer)
            zd_aon  = MAX ( zd_aon1, zd_aon2 )

            zd_eon1 = ( lys_N(layer) - rem_N(layer) ) * ddtb
            zd_eon2 = - cbu_i_bio(jn_eon,layer)
            zd_eon  = MAX ( zd_eon1, zd_eon2 )

            !--- Correct rates to prevent leaks
            IF ( zd_din .EQ. zd_din2 )
     &         syn_N(layer) = MIN( syn_N(layer), -zd_din2/ddtb)

            IF ( zd_aon .EQ. zd_aon2 )
     &         lys_N(layer) = MIN( lys_N(layer), -zd_aon2/ddtb)

            IF ( zd_eon .EQ. zd_eon2 )
     &         rem_N(layer) = MIN( rem_N(layer), -zd_eon2/ddtb)


         ELSE
            zd_din1 = no3_c * ( rem_bio(layer) +
     &                rsp_bio(layer) - syn_bio(layer) ) * ddtb
            zd_din2 = - cbu_i_bio(jn_din,layer)        ! Max NO3 uptake
            zd_din  = MAX ( zd_din1, zd_din2 )     
             
         ENDIF ! ln_decoupNC

!!!!!!!!!!!!!!!!!!

         IF (ln_decoupPC) THEN

            !--- Increments
            zd_dip1 = ( rsp_P(layer) - syn_P(layer) +
     &                  rem_P(layer) ) * ddtb
            zd_dip2 = - cbu_i_bio(jn_dip,layer) !PEU PAS EPUISER PLUS
                                                !QUE CE QUIL Y A !!!!
            zd_dip  = MAX ( zd_dip1, zd_dip2 )  ! interessant dans le
                                                ! ou zd_din1 est négatif
                                                ! zd_din1 peut pas être
                                                ! plus petit que zd_din2 

            zd_aop1 = ( syn_P(layer) - rsp_P(layer) - lys_P(layer) ) *
     &                ddtb
            zd_aop2 = - cbu_i_bio(jn_aop,layer)
            zd_aop  = MAX ( zd_aop1, zd_aop2 )

            zd_eop1 = ( lys_P(layer) - rem_P(layer) ) * ddtb
            zd_eop2 = - cbu_i_bio(jn_eop,layer)
            zd_eop  = MAX ( zd_eop1, zd_eop2 )

            !--- Correct rates to prevent leaks
            IF ( zd_dip .EQ. zd_dip2 )
     &         syn_P(layer) = MIN( syn_P(layer), -zd_dip2/ddtb)

            IF ( zd_aop .EQ. zd_aop2 )
     &         lys_P(layer) = MIN( lys_P(layer), -zd_aop2/ddtb)

            IF ( zd_eop .EQ. zd_eop2 )
     &         rem_P(layer) = MIN( rem_P(layer), -zd_eop2/ddtb)


         ELSE
            zd_dip1 = po4_c * ( rem_bio(layer) +
     &                rsp_bio(layer) - syn_bio(layer) ) * ddtb
            zd_dip2 = - cbu_i_bio(jn_dip,layer)        ! Max DIP uptake
            zd_dip  = MAX ( zd_dip1, zd_dip2 )

         ENDIF ! ln_decoupPC


        ! -------------------------------------------------------
        ! -- 2) Add Increments
        ! ------------------------------------------------------

         IF (ln_decoupNC) THEN
            cbu_i_bio(jn_din,layer) = cbu_i_bio(jn_din,layer) + zd_din
            cbu_i_bio(jn_aon,layer) = cbu_i_bio(jn_aon,layer) + zd_aon
            cbu_i_bio(jn_eon,layer) = cbu_i_bio(jn_eon,layer) + zd_eon
         ELSE
            cbu_i_bio(jn_din,layer) = cbu_i_bio(jn_din,layer) + zd_din
         ENDIF

         IF (ln_decoupPC) THEN
            cbu_i_bio(jn_dip,layer) = cbu_i_bio(jn_dip,layer) + zd_dip
            cbu_i_bio(jn_aop,layer) = cbu_i_bio(jn_aop,layer) + zd_aop
            cbu_i_bio(jn_eop,layer) = cbu_i_bio(jn_eop,layer) + zd_eop
         ELSE
            cbu_i_bio(jn_dip,layer) = cbu_i_bio(jn_dip,layer) + zd_dip
         ENDIF


         cbu_i_bio(jn_dsi,layer) = cbu_i_bio(jn_dsi,layer) + zd_dsi
         cbu_i_bio(jn_aoc,layer) = cbu_i_bio(jn_aoc,layer) + zd_daf

         IF ( nn_bio_opt .GE. 1 ) 
     &      cbu_i_bio(jn_eoc,layer) = cbu_i_bio(jn_eoc,layer) + zd_eoc


         IF ( ln_write_bio ) THEN

            WRITE(numout,*) ' Terms for layer: ', layer
            WRITE(numout,*) ' ~~~~~~~~~~~~~~~~ '
            WRITE(numout,*) ' syn_bio   : ', syn_bio(layer)
            WRITE(numout,*) ' lys_bio   : ', lys_bio(layer)
            WRITE(numout,*) ' rem_bio   : ', rem_bio(layer)
            WRITE(numout,*) ' exu_bio   : ', exu_bio(layer)
            WRITE(numout,*) ' rsp_bio   : ', rsp_bio(layer)
            WRITE(numout,*) ' lim_dsi   : ', lim_dsi(layer)
            WRITE(numout,*) ' lim_no3   : ', lim_no3(layer)
            WRITE(numout,*) ' lim_po4   : ', lim_po4(layer)
            WRITE(numout,*) ' lim_lig   : ', lim_lig(layer)
            WRITE(numout,*) ' lim_tem   : ', lim_tem(layer)
            WRITE(numout,*) ' lim_sal   : ', lim_sal(layer)
            WRITE(numout,*) ' lim_din_stock : ', lim_din_stock(layer)
            WRITE(numout,*) ' lim_dip_stock : ', lim_dip_stock(layer)
            WRITE(numout,*) ' lim_dsi_stock : ', lim_dsi_stock(layer)
            WRITE(numout,*)
            WRITE(numout,*) ' tc_bio    : ', tc_bio(layer)  
            WRITE(numout,*) ' sbr_bio   : ', sbr_bio(layer)  
            WRITE(numout,*) ' par_bio   : ', par_bio(layer)
            WRITE(numout,*)
            WRITE(numout,*) ' zd_dsi    : ', zd_dsi
            WRITE(numout,*) ' zd_po4    : ', zd_po4
            WRITE(numout,*) ' zd_daf    : ', zd_daf
            WRITE(numout,*) ' zd_eoc    : ', zd_eoc
            WRITE(numout,*) ' zd_din    : ', zd_din
            WRITE(numout,*)
            WRITE(numout,*) ' dSi       : ', cbu_i_bio(jn_dsi,layer)
            WRITE(numout,*) ' dIN       : ', cbu_i_bio(jn_din,layer) 
            WRITE(numout,*) ' dIP       : ', cbu_i_bio(jn_dip,layer)
            WRITE(numout,*) ' AoC       : ', cbu_i_bio(jn_aoc,layer)
            WRITE(numout,*) ' eoC       : ', cbu_i_bio(jn_eoc,layer)
            WRITE(numout,*) ' krem_N    : ', krem_N

        
            IF (ln_decoupNC) THEN 
               WRITE(numout,*) ' syn_N    : ', syn_N(layer)
               WRITE(numout,*) ' rsp_N    : ', rsp_N(layer)
               WRITE(numout,*) ' lys_N    : ', lys_N(layer)
               WRITE(numout,*) ' rem_N    : ', rem_N(layer)
               WRITE(numout,*)
               WRITE(numout,*) ' zd_aon    : ', zd_aon
               WRITE(numout,*) ' zd_eon    : ', zd_eon
               WRITE(numout,*)
               WRITE(numout,*) ' AoN       : ', cbu_i_bio(jn_aon,layer)
               WRITE(numout,*) ' eoN       : ', cbu_i_bio(jn_eon,layer)
               WRITE(numout,*)
               WRITE(numout,*) ' N_C_alg   : ', N_C_alg(layer)
               WRITE(numout,*) ' N_C_det   : ', N_C_det(layer)
            ENDIF ! ln_decoupNC

         ENDIF ! ln_write_bio

      END DO ! layer
   


      !------------------------------------------------------------------------
      ! 2.5 Update DIC, Alk and Oxy
      !------------------------------------------------------------------------

      DO layer = layer_00, nlay_bio

         zncp  =  - syn_bio(layer) + rsp_bio(layer) + rem_bio(layer)

         IF ( ln_carbon .AND. flag_active(jn_dic) 
     &                  .AND. flag_active(jn_alk) ) THEN

            zd_dic1   = - zncp * ddtb                  !! Bizarre
            zd_dic2   = - cbu_i_bio(jn_dic,layer)
            zd_dic    = MAX( zd_dic1 , zd_dic2 )

            zd_alk1   =   no3_c * zncp * ddtb           !! Bizarre
            zd_alk2   = - cbu_i_bio(jn_alk,layer)
            zd_alk    = MAX( zd_alk1 , zd_alk2 )

            cbu_i_bio(jn_dic,layer) = cbu_i_bio(jn_dic,layer) + zd_dic 
            cbu_i_bio(jn_alk,layer) = cbu_i_bio(jn_alk,layer) + zd_alk 

            IF ( ln_write_bio ) THEN
               WRITE(numout,*) ' DIC       : ', cbu_i_bio(jn_dic,layer)
               WRITE(numout,*) ' Alk       : ', cbu_i_bio(jn_alk,layer)
            ENDIF

         ENDIF

         IF ( flag_active(jn_oxy) ) THEN

            zd_oxy1   =   oxy_c * zncp * ddtb
            zd_oxy2   = - cbu_i_bio(jn_oxy,layer)
            zd_oxy    = MAX( zd_oxy1, zd_oxy2 )

            cbu_i_bio(jn_oxy,layer) = cbu_i_bio(jn_oxy,layer) + zd_oxy

            IF ( ln_write_bio ) THEN
               WRITE(numout,*) ' Oxy       : ', cbu_i_bio(jn_oxy,layer)
            ENDIF

         ENDIF

      END DO

!
!------------------------------------------------------------------------------!
! X) End of the routine
!------------------------------------------------------------------------------!
!
      WRITE(numout,*)
      WRITE(numout,*) '    *** After biological sources and sinks *** '
      WRITE(numout,*) '    model output '
      DO jn = 1, ntra_bio
         IF ( flag_active(jn) ) THEN
           WRITE(numout,*) ' biotr_i_nam : ', biotr_i_nam(jn)
           WRITE(numout,*) ' cbu_i_bio : ', ( cbu_i_bio(jn, jk), jk = 1,
     &                                      nlay_bio )
         ENDIF
      END DO
      WRITE(numout,*)

      WRITE(numout,*)
      WRITE(numout,*) ' End of ice_bio_sms '
      WRITE(numout,*) '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
     

!==============================================================================|
! Fin de la routine ice_bio_sms
      WRITE(numout,*)

      END
